@@ -30,7 +30,7 @@ int start_socket_client(char *host, char *path, int argc, char *argv[])
 	int fd, i;
 	char *sargs[MAX_ARGS];
 	int sargc=0;
-	char line[1024];
+	char line[MAXPATHLEN];
 	char *p, *user=NULL;
 	extern int remote_version;
 
@@ -102,13 +102,12 @@ static int rsync_module(int fd, int i)
 	int argc=0;
 	char *argv[MAX_ARGS];
 	char **argp;
-	char line[1024];
-	uid_t uid;
-	gid_t gid;
+	char line[MAXPATHLEN];
+	uid_t uid = (uid_t)-2;
+	gid_t gid = (gid_t)-2;
 	char *p;
 	char *addr = client_addr(fd);
 	char *host = client_name(fd);
-	char *auth;
 	char *name = lp_name(i);
 	int start_glob=0;
 
@@ -134,8 +133,6 @@ static int rsync_module(int fd, int i)
 		return -1;
 	}
 
-	rprintf(FINFO,"rsync on module %s from %s (%s)\n",
-		name, host, addr);
 	
 	module_id = i;
 
@@ -168,22 +165,28 @@ static int rsync_module(int fd, int i)
 	p = lp_exclude(i);
 	add_exclude_line(p);
 
+	log_open();
+
 	if (chroot(lp_path(i))) {
+		rprintf(FERROR,"chroot %s failed\n", lp_path(i));
 		io_printf(fd,"@ERROR: chroot failed\n");
 		return -1;
 	}
 
 	if (chdir("/")) {
+		rprintf(FERROR,"chdir %s failed\n", lp_path(i));
 		io_printf(fd,"@ERROR: chdir failed\n");
 		return -1;
 	}
 
-	if (setgid(gid)) {
+	if (setgid(gid) || getgid() != gid) {
+		rprintf(FERROR,"setgid %d failed\n", gid);
 		io_printf(fd,"@ERROR: setgid failed\n");
 		return -1;
 	}
 
-	if (setuid(uid)) {
+	if (setuid(uid) || getuid() != uid) {
+		rprintf(FERROR,"setuid %d failed\n", uid);
 		io_printf(fd,"@ERROR: setuid failed\n");
 		return -1;
 	}
@@ -201,18 +204,18 @@ static int rsync_module(int fd, int i)
 
 		p = line;
 
-		if (start_glob && strncmp(p, name, strlen(name)) == 0) {
-			p += strlen(name);
-			if (!*p) p = ".";
-		}
-
 		argv[argc] = strdup(p);
 		if (!argv[argc]) {
 			return -1;
 		}
 
 		if (start_glob) {
-			glob_expand(argv, &argc, MAX_ARGS);
+			if (start_glob == 1) {
+				rprintf(FINFO,"rsync on %s from %s (%s)\n",
+					p, host, addr);
+				start_glob++;
+			}
+			glob_expand(name, argv, &argc, MAX_ARGS);
 		} else {
 			argc++;
 		}
@@ -263,7 +266,7 @@ static int start_daemon(int fd)
 	extern char *config_file;
 	extern int remote_version;
 
-	if (!lp_load(config_file)) {
+	if (!lp_load(config_file, 0)) {
 		exit_cleanup(1);
 	}
 
@@ -324,6 +327,8 @@ static int start_daemon(int fd)
 
 int daemon_main(void)
 {
+	extern char *config_file;
+
 	if (is_a_socket(STDIN_FILENO)) {
 		/* we are running via inetd */
 		return start_daemon(STDIN_FILENO);
@@ -331,6 +336,15 @@ int daemon_main(void)
 
 	become_daemon();
 
+	if (!lp_load(config_file, 1)) {
+		fprintf(stderr,"failed to load config file %s\n", config_file);
+		exit_cleanup(1);
+	}
+
+	log_open();
+
+	rprintf(FINFO,"rsyncd version %s starting\n",VERSION);
+
 	start_accept_loop(rsync_port, start_daemon);
 	return -1;
 }
